var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _class, _temp2;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React, { Component } from 'react';
import moment from 'moment';
import PropTypes from 'prop-types';
import GanttBar from './GanttBar';
import GanttPopup from './GanttPopup';

var GanttRow = (_temp2 = _class = function (_Component) {
  _inherits(GanttRow, _Component);

  function GanttRow() {
    var _temp, _this, _ret;

    _classCallCheck(this, GanttRow);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, _Component.call.apply(_Component, [this].concat(args))), _this), _this.state = {
      active: false,
      mouse: {},
      activeStep: {},
      markerTime: moment().toDate()
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  GanttRow.prototype.renderPopup = function renderPopup() {
    var _props = this.props,
        popupStyle = _props.popupStyle,
        title = _props.title;
    var _state = this.state,
        activeStep = _state.activeStep,
        markerTime = _state.markerTime,
        active = _state.active;

    if (_.isEmpty(activeStep)) return React.createElement('div', null);
    return React.createElement(
      'div',
      { style: {
          position: 'absolute',
          left: this.state.mouse.offsetX + 'px',
          display: active ? 'inherit' : 'none'
        } },
      React.createElement(GanttPopup, {
        style: popupStyle,
        title: title,
        activeStep: activeStep,
        markerTime: markerTime
      })
    );
  };

  GanttRow.prototype.render = function render() {
    var _props2 = this.props,
        title = _props2.title,
        markerStyle = _props2.markerStyle,
        templateName = _props2.templateName,
        steps = _props2.steps;
    var active = this.state.active;

    var tdStyle = { whiteSpace: 'nowrap' };

    var _calculateBarStyle = this.calculateBarStyle(this.props.barStyle),
        barStyle = _calculateBarStyle.barStyle,
        barWrapperStyle = _calculateBarStyle.barWrapperStyle;

    return React.createElement(
      'tr',
      { style: { cursor: 'inherit' } },
      React.createElement(
        'td',
        { style: _extends({}, tdStyle, {
            width: '0px'
          }) },
        title
      ),
      React.createElement(
        'td',
        { style: _extends({}, tdStyle, {
            width: '100%'
          }) },
        React.createElement(
          'div',
          { style: barWrapperStyle },
          React.createElement(GanttBar, {
            title: title,
            templateName: templateName,
            steps: steps,
            style: barStyle
          }),
          React.createElement('div', { style: _extends({}, markerStyle, {
              height: barStyle.height,
              marginTop: '-' + barStyle.height,
              position: 'relative',
              marginLeft: this.state.mouse.offsetX - parseInt(markerStyle.width) / 2 + 'px',
              zIndex: 0,
              display: active ? 'inherit' : 'none'
            }) }),
          React.createElement('div', {
            style: {
              height: barStyle.height,
              marginTop: '-' + barStyle.height,
              position: 'relative',
              zIndex: 0
            },
            onMouseEnter: this.handleMouseEnter.bind(this),
            onMouseLeave: this.handleMouseLeave.bind(this)
          })
        ),
        this.renderPopup()
      )
    );
  };

  GanttRow.prototype.handleMouseEnter = function handleMouseEnter() {
    var _this2 = this;

    this.setState({ active: true });
    this.mouseEventListener = window.addEventListener('mousemove', function (e) {
      return _this2.handleMouseMove(e);
    });
  };

  GanttRow.prototype.handleMouseLeave = function handleMouseLeave() {
    this.setState({ active: false });
    if (this.mouseEventListener) {
      this.mouseEventListener.removeEventListener();
    }
  };

  GanttRow.prototype.handleMouseMove = function handleMouseMove(e) {
    if (this.state.active) {
      var markerStyle = this.props.markerStyle;
      var _context = this.context,
          timelineWidth = _context.timelineWidth,
          leftBound = _context.leftBound;

      var markerTime = moment(leftBound).add(this.widthToDuration(e.offsetX), 'seconds').toDate();
      this.setState({
        mouse: e,
        markerTime: markerTime,
        activeStep: this.getStepFromTime(markerTime)
      });
    }
  };

  GanttRow.prototype.calculateBarStyle = function calculateBarStyle(barStyle) {
    barStyle = _.clone(barStyle);
    var margin = this.getMargin(barStyle.margin);
    var marginTop = barStyle.marginTop || margin.marginTop;
    var marginBottom = barStyle.marginBottom || margin.marginBottom;
    delete barStyle.marginTop;
    delete barStyle.marginBottom;
    delete barStyle.margin;
    return {
      barStyle: barStyle,
      barWrapperStyle: {
        marginTop: marginTop,
        marginBottom: marginBottom
      }
    };
  };

  GanttRow.prototype.getMargin = function getMargin(margin) {
    var marginTop = '0px';
    var marginRight = '0px';
    var marginBottom = '0px';
    var marginLeft = '0px';
    margin = margin ? margin.split(' ') : [];
    switch (margin.length) {
      case 1:
        marginTop = margin[0];
        marginRight = margin[0];
        marginBottom = margin[0];
        marginLeft = margin[0];
      case 2:
        marginTop = margin[0];
        marginRight = margin[1];
        marginBottom = margin[0];
        marginLeft = margin[1];
      case 4:
        marginTop = margin[0];
        marginRight = margin[1];
        marginBottom = margin[2];
        marginLeft = margin[3];
    }
    return {
      marginTop: marginTop,
      marginRight: marginRight,
      marginBottom: marginBottom,
      marginLeft: marginLeft
    };
  };

  GanttRow.prototype.widthToDuration = function widthToDuration(width) {
    var _context2 = this.context,
        leftBound = _context2.leftBound,
        rightBound = _context2.rightBound,
        timelineWidth = _context2.timelineWidth;

    var timelineDuration = moment(rightBound).diff(leftBound, 'seconds');
    var pixelPerSecond = timelineDuration / timelineWidth;
    return pixelPerSecond * width;
  };

  GanttRow.prototype.getStepFromTime = function getStepFromTime(time) {
    var _props3 = this.props,
        steps = _props3.steps,
        templateName = _props3.templateName;
    var templates = this.context.templates;

    var templateStep = {};
    var templateSteps = templates[templateName].steps;
    _.each(steps, function (step, index) {
      if (moment(time).isAfter(step) && moment(time).isBefore(steps[index + 1])) {
        templateStep = templateSteps[index];
        return false;
      }
    });
    return templateStep;
  };

  return GanttRow;
}(Component), _class.contextTypes = {
  templates: PropTypes.object.isRequired,
  dateFormat: PropTypes.string.isRequired,
  leftBound: PropTypes.object.isRequired,
  rightBound: PropTypes.object.isRequired,
  timelineWidth: PropTypes.number.isRequired,
  debug: PropTypes.bool.isRequired
}, _class.defaultProps = {
  barStyle: {
    height: '80px',
    marginTop: '10px',
    marginBottom: '10px'
  },
  popupStyle: {
    backgroundColor: '#FFFFFF',
    padding: '20px',
    boxShadow: '0px 0px 15px 0px rgba(0,0,0,0.75)'
  },
  markerStyle: {
    width: '40px',
    backgroundColor: '#000000',
    opacity: 0.5
  },
  style: {},
  templateName: 'default',
  title: '',
  children: null
}, _temp2);
export { GanttRow as default };
GanttRow.propTypes = process.env.NODE_ENV !== "production" ? {
  barStyle: PropTypes.object,
  popupStyle: PropTypes.object,
  markerStyle: PropTypes.object,
  steps: PropTypes.array.isRequired,
  style: PropTypes.object,
  templateName: PropTypes.string,
  title: PropTypes.string,
  children: PropTypes.node
} : {};